from .download import download_file
from pathlib import Path
download_file("addons_file/BB_Nodes/BB.blend",Path(__file__).parent)
bl_info = {
    "name" : "BB Nodes",
    "author" : "Blender Bash",
    "description" : "Custom made Node groups that improve and speed up procedural modeling with Geometry nodes",
    "blender" : (4, 0, 0),
    "version" : (1, 4, 2),
    "location" : "Geometry Nodes Editor > Add > BB",
    "warning" : "When installing a new version, first uninstall the old one and restart Blender",
    "category" : "Node"
}
import json
import bpy
import os
import re
from bpy.types import (
    Operator,
    Menu,
)
from bpy.props import (
    StringProperty,
    EnumProperty,
    BoolProperty,
    PointerProperty
)


def add_bb_button(self, context):
    if context.area.ui_type == 'GeometryNodeTree':
        self.layout.menu('NODE_MT_bb_geo_menu', text="BB Nodes", icon='FORWARD')


geo_node_group = {}
cat_list = []

dir_path = os.path.dirname(__file__)

# adapted code from https://github.com/blender/blender/blob/master/release/scripts/modules/nodeitems_utils.py
def cat_generator():
    global cat_list
    cat_list = []
    for item in geo_node_group.items():
        def my_list(self,context):
            layout = self.layout
            for name_group in geo_node_group[self.bl_label]:
                props = layout.operator(
                    NODE_OT_group_add.bl_idname,
                    text=re.sub(r'.*?_', '', name_group), # filter prefix
                )
                props.name_group = name_group

        menu_type = type("NODE_MT_group_" + item[0], (bpy.types.Menu,), {
            "bl_idname": "NODE_MT_group_" + item[0].replace(" ", "_"),   # replace whitespace with uderscore to avoid alpha-numeric suffix warning 
            "bl_space_type": 'NODE_EDITOR',
            "bl_label": item[0],
            "draw": my_list,
        })
        if menu_type not in cat_list:
            def generate_menu_draw(name,label): # Wrapper function to force unique references
                def draw_menu(self,context):
                    self.layout.menu(name, text=label)
                return draw_menu
            bpy.utils.register_class(menu_type)
            bpy.types.NODE_MT_bb_geo_menu.append(generate_menu_draw(menu_type.bl_idname,menu_type.bl_label))
            cat_list.append(menu_type)


class NODE_MT_bb_geo_menu(Menu):
    bl_label = "BB nodes"
    bl_idname = 'NODE_MT_bb_geo_menu'

    @classmethod
    def poll(cls,context):
        return context.space_data.tree_type == 'GeometryNodeTree'

    def draw(self, context):
        pass

class NODE_OT_group_add(Operator):
    """Add a node group"""
    bl_idname = "bb." + os.path.basename(dir_path).lower()
    bl_label = "Add node group"
    bl_description = "Append Node Group"
    bl_options = {'REGISTER', 'UNDO'}

    name_group: StringProperty()

    @classmethod
    def poll(cls,context):
        return context.space_data.node_tree

    def execute(self, context):
        old_groups = set(bpy.data.node_groups)
        
        for file in os.listdir(dir_path):
            if file.endswith(".blend"):
                filepath = os.path.join(dir_path,file)
                break
        else:
            raise FileNotFoundError("No .blend File in directory "+ dir_path)
                
        with bpy.data.libraries.load(filepath, link=False) as (data_from, data_to):
            if self.name_group not in bpy.data.node_groups:
                data_to.node_groups.append(self.name_group)
        new_groups = list(set(bpy.data.node_groups)-old_groups)
        for group in new_groups:
            for node in group.nodes:
                if node.type == "GROUP":
                    name_new = node.node_tree.name.split(".")[0]
                    node.node_tree = bpy.data.node_groups[name_new]
        for group in new_groups:
            if "." in group.name:
                bpy.data.node_groups.remove(group) 

        bpy.ops.node.add_node(type="GeometryNodeGroup")
            
        node = context.selected_nodes[0]
        node.node_tree = bpy.data.node_groups[self.name_group]
        bpy.ops.transform.translate('INVOKE_DEFAULT')
        
        return {'FINISHED'}

def search_group(self,context):
        for prop in context.scene.use_render:
            if prop.n_tree == context.space_data.node_tree:
                return prop
            

class override_use_render(bpy.types.Operator):
    bl_idname = "bb.override_use_render"
    bl_label = "Override Show Render"
    bl_description = "Overrieds all 'Show Render' Checkboxes in the Current Node Tree"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        space = context.space_data
        context.scene.use_render.add().n_tree = space.node_tree
        return {'FINISHED'}

class NODE_PT_BB_Options(bpy.types.Panel):
    bl_idname = "NODE_PT_BB_Options"
    bl_label = "BB Options"
    bl_region_type = "UI"
    bl_space_type = "NODE_EDITOR"
    bl_category = "Options"

    @classmethod
    def poll(cls, context):
        return context.space_data.node_tree != None

    def draw(self, context):
        if not context.scene.use_render:
            self.layout.operator("bb.override_use_render")
        else: 
            for prop in context.scene.use_render:
                if prop.n_tree == context.space_data.node_tree:
                    break
            else:
                self.layout.operator("bb.override_use_render")

class NODE_PT_BB_Options_override(bpy.types.Panel):
    bl_parent_id = "NODE_PT_BB_Options"
    bl_label = ""
    bl_region_type = "UI"
    bl_space_type = "NODE_EDITOR"

    @classmethod
    def poll(cls, context):
        if context.scene.use_render:
            for prop in context.scene.use_render:
                if prop.n_tree == context.space_data.node_tree:
                    return True
            else:
                return False
    
    def draw_header(self,context):
        property_group = search_group(self,context)
        self.layout.prop(property_group,"use_render")

    def draw(self, context):
        property_group = search_group(self,context)
        self.layout.prop(property_group,"value")


def update_use_render(self,context):
    property_group = search_group(self, context)
    override = property_group.value
    switch = property_group.use_render
    
    for node in context.space_data.node_tree.nodes:
        if "Show Render" in node.inputs:
            if switch:
                node['show_render_backup'] = node.inputs['Show Render'].default_value
            else:
                node.inputs['Show Render'].default_value = node['show_render_backup']
    update_name(self, context)


def update_name(self,context):
    property_group = search_group(self, context)
    override = property_group.value
    switch = property_group.use_render
    
    for node in context.space_data.node_tree.nodes:
        if "Show Render" in node.inputs:
            if switch:
                node.inputs['Show Render'].default_value = override

class use_render_props(bpy.types.PropertyGroup):
    use_render: BoolProperty(name="Override Show Render", default=False,update=update_use_render)
    value: BoolProperty(name="Show Render", default=False,update=update_name)
    n_tree: PointerProperty(type=bpy.types.NodeTree)



classes = (
    NODE_OT_group_add,
    NODE_PT_BB_Options,
    override_use_render,
    use_render_props,
    NODE_PT_BB_Options_override,
    )


def register():
    global geo_node_group

    with open(os.path.join(os.path.dirname(__file__), "geo_nodes.json"), 'r') as f:
        geo_node_group = json.loads(f.read())

    if not hasattr(bpy.types, "NODE_MT_bb_geo_menu"):
        bpy.utils.register_class(NODE_MT_bb_geo_menu)
        bpy.types.NODE_MT_add.append(add_bb_button)
    for cls in classes:
        bpy.utils.register_class(cls)

    cat_generator()

    bpy.types.Scene.use_render = bpy.props.CollectionProperty(
        type=use_render_props)

def unregister():
    if hasattr(bpy.types, "NODE_MT_bb_geo_menu"):
        bpy.types.NODE_MT_add.remove(add_bb_button)
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.use_render

if __name__ == "__main__":
    register()
